package com.genpt.nsight.site

import org.apache.commons.cli.{DefaultParser, Option, Options}
import org.apache.hadoop.conf.Configuration
import org.apache.hadoop.fs.{FileSystem, Path}
import org.apache.spark.sql.SparkSession

import java.time.format.DateTimeFormatter
import java.time.{ZoneOffset, ZonedDateTime}

object SiteRelationshipExport extends App {

  def getCurrentTimeInUTC(now: ZonedDateTime, epoch: Boolean): String = {
    if (epoch) {
      now.toEpochSecond.toString
    } else {
      val formatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmss")
      formatter.format(now)
    }
  }

  def getexportquery(location: String, relationshiptype: String): String = {
    val aliasNames = "'9 Digit Store Number','DC Abbreviation Code','Supplier Abbreviation Location'"
    if (location == null) {
      if (relationshiptype == null) {
        println(s"Empty location and relationshipType found, so running export for all the locations and relationship types")
        s"""
        SELECT
          site.alias_value as siteAlias,
          concat(site.alias_value,'-',ctry.alpha_2_code,'-',count(1) OVER (PARTITION BY sr.site_identifier)),
          sr.relationship_type as relationshipType,
          relatedSite.alias_value as relatedSiteAlias,
          sr.related_site_type as relatedSiteType,
          sr.relationship_rank as relationshipRank,
          unix_timestamp(sr.data_as_of_ts) as dataAsOfTs
        FROM nsight.$tableName sr
        INNER JOIN nsight.site site ON sr.site_identifier=site.site_identifier AND site.alias_name IN ($aliasNames)
        INNER JOIN nsight.site relatedSite ON sr.related_site_identifier=relatedSite.site_identifier AND relatedSite.alias_name IN ($aliasNames)
        INNER JOIN nsight.country_code as ctry ON ctry.alpha_3_code = site.country
        """
      }
      else {
        println(s"Empty location found, so running export for given relationship types $relationshiptype")
        s"""
        SELECT
          site.alias_value as siteAlias,
          concat(site.alias_value,'-',ctry.alpha_2_code,'-',count(1) OVER (PARTITION BY sr.site_identifier)),
          sr.relationship_type as relationshipType,
          relatedSite.alias_value as relatedSiteAlias,
          sr.related_site_type as relatedSiteType,
          sr.relationship_rank as relationshipRank,
          unix_timestamp(sr.data_as_of_ts) as dataAsOfTs
        FROM nsight.$tableName sr
        INNER JOIN nsight.site site ON sr.site_identifier=site.site_identifier AND site.alias_name IN ($aliasNames)
        INNER JOIN nsight.site relatedSite ON sr.related_site_identifier=relatedSite.site_identifier AND relatedSite.alias_name IN ($aliasNames)
        INNER JOIN nsight.country_code as ctry ON ctry.alpha_3_code = site.country
        WHERE sr.relationship_type in ($relationshiptype)
        """
      }
    } else {
      if (relationshiptype == null) {
        println(s"Empty relationship type found, so running export for all relationshiptypes for the given location(s) $location")
        s"""
        SELECT
          site.alias_value as siteAlias,
          concat(site.alias_value,'-',ctry.alpha_2_code,'-',count(1) OVER (PARTITION BY sr.site_identifier)),
          sr.relationship_type as relationshipType,
          relatedSite.alias_value as relatedSiteAlias,
          sr.related_site_type as relatedSiteType,
          sr.relationship_rank as relationshipRank,
          unix_timestamp(sr.data_as_of_ts) as dataAsOfTs
        FROM nsight.$tableName sr
        INNER JOIN nsight.site site ON sr.site_identifier=site.site_identifier AND site.alias_name IN ($aliasNames)
        INNER JOIN nsight.site relatedSite ON sr.related_site_identifier=relatedSite.site_identifier AND relatedSite.alias_name IN ($aliasNames)
        INNER JOIN nsight.country_code as ctry ON ctry.alpha_3_code = site.country
        WHERE site.alias_value IN ($location)
        """
      } else { //Export sitetosite data for the given location(s) and relationshiptype(s)
        println(s"Running exports for the given location(s) $location and relationship type(s) $relationshiptype")
        s"""
        SELECT
          site.alias_value as siteAlias,
          concat(site.alias_value,'-',ctry.alpha_2_code,'-',count(1) OVER (PARTITION BY sr.site_identifier)),
          sr.relationship_type as relationshipType,
          relatedSite.alias_value as relatedSiteAlias,
          sr.related_site_type as relatedSiteType,
          sr.relationship_rank as relationshipRank,
          unix_timestamp(sr.data_as_of_ts) as dataAsOfTs
        FROM nsight.$tableName sr
        INNER JOIN nsight.site site ON sr.site_identifier=site.site_identifier AND site.alias_name IN ($aliasNames)
        INNER JOIN nsight.site relatedSite ON sr.related_site_identifier=relatedSite.site_identifier AND relatedSite.alias_name IN ($aliasNames)
        INNER JOIN nsight.country_code as ctry ON ctry.alpha_3_code = site.country
        WHERE site.alias_value IN ($location) and sr.relationship_type in ($relationshiptype)
        """
      }
    }
  }

  def generatefilebylocation(outputpath: String, tblname: String, hdfs: FileSystem, f: Path) = {
    //      Getting file directory path and converting to string e.g. f=/StoreInvExport/store_inventory_20220909.csv/location=400006548/part-r-00000-918ba26e-35f5-4458-b7f1-68b3e8bd4b1d.csv then get /StoreInvExport/store_inventory_20220909.csv/location=400006548
    val str = f.getParent.toString
    //      Splitting the file path based on the delimiter "=" to get the location and fetch the second element e.g StoreInvExport/store_inventory_20220909.csv/location=400006548 will be split into StoreInvExport/store_inventory_20220909.csv/location and 400006548
    val Array(path, partition) = str.split("=")
    val Array(loc, country, count) = partition.split("-")
    //      Generating the target filename store_inventory_<<country>>_<<location>>_<<count>>_<<date>>.csv
    val outputfilename = tblname + "_" + country + "_" + loc + "_" + count + "_" + day + ".csv"
    //      Defining the Source Path and Destination Path to copy the output files from dse to local
    //      https://stackoverflow.com/questions/48226882/move-file-from-one-folder-to-another-on-hdfs-in-scala-spark
    val srcPath = new Path(s"${f}")
    val destPath = new Path(s"file://${outputpath}/${outputfilename}")
    println(s"Copying ${srcPath} to ${destPath}")
    hdfs.copyToLocalFile(false, srcPath, destPath, true)
  }

  val LOCATION_OPTION = "locations"
  val LOCATIONTYPE_OPTION = "locationtype"
  val RELATIONSHIPTYPE_OPTION = "relationshiptype"
  val OUTPUTPATH_OPTION = "outputpath"
    val tableName = "site_relationship"
  var now: ZonedDateTime = ZonedDateTime.now(ZoneOffset.UTC)
  var day: String = null
  val HELP_OPTION = "help"

  try {
    var location: String = null
    var locationtype: String = null
    var relationshiptype: String = null
    var outputpath: String = null
    var recordCount: Long = 0
    var options: Options = new Options()

    options.addOption(Option.builder("l")
      .longOpt(LOCATION_OPTION)
      .desc("location(s) to export")
      .hasArg()
      .required(false)
      .argName("location")
      .build())
    options.addOption(Option.builder("lt")
      .longOpt(LOCATIONTYPE_OPTION)
      .desc("location type to export")
      .hasArg().
      required(false).
      argName("locationtype")
      .build())
    options.addOption(Option.builder("rt")
      .longOpt(RELATIONSHIPTYPE_OPTION)
      .desc("relationship type to export")
      .hasArg().
      required(false).
      argName("relationshiptype")
      .build())
    options.addOption(Option.builder("o")
      .longOpt(OUTPUTPATH_OPTION)
      .desc("output file path on the local system")
      .hasArg().
      required(true).
      argName("outputpath")
      .build())
    options.addOption(Option.builder("h")
      .longOpt(HELP_OPTION)
      .build())

      val cmd = new DefaultParser().parse(options, args)

    if (cmd.hasOption(LOCATION_OPTION)) {
      println(s"Location Option is provided as " + cmd.getOptionValue(LOCATION_OPTION))
      location = cmd.getOptionValue(LOCATION_OPTION).split(",").map("'" + _ + "'").mkString(",")
    }
    if (cmd.hasOption(RELATIONSHIPTYPE_OPTION)) {
      relationshiptype = cmd.getOptionValue(RELATIONSHIPTYPE_OPTION).split(",").map("'" + _ + "'").mkString(",")
    }
    if (cmd.hasOption(OUTPUTPATH_OPTION)) {
      outputpath = cmd.getOptionValue(OUTPUTPATH_OPTION)
    }

    def clsname = getClass.getSimpleName.split('$').head

    val spark = SparkSession.builder().appName("SiteRelationshipExportToGCP").getOrCreate()
    var SnapshotDF = spark.emptyDataFrame

    // Fetch Query to run
    val query = getexportquery(location, relationshiptype)
    SnapshotDF = spark.sqlContext.sql(query).toDF("location", "locationforpartition", "relationshipType", "relatedLocation", "relatedSiteType", "relationshipRank", "dataAsOfTs")
    day = getCurrentTimeInUTC(now = now, false)
    val filename = tableName + "_" + day

    recordCount = SnapshotDF.count()
    println("Fetched " + recordCount + " SiteRelationship records")

    if (recordCount > 0) {
      //  Generating the output file and overwriting the content if any on the dse fs and splitting the output file by location
      println(s"Writing the output to dse ${filename}")
      SnapshotDF.repartition(1).write.mode("overwrite").partitionBy("locationforpartition").csv("/" + clsname + "/" + filename)

      //  Copying file from DSEFS to local system
      val fs = FileSystem.get(spark.sparkContext.hadoopConfiguration)
      val hadoopConf = new Configuration()
      val hdfs = FileSystem.get(hadoopConf)
      val file = fs.globStatus(new Path("/" + clsname + "/" + filename + "/locationforpartition*/part*"))
      val filelist = file.map(f => f.getPath)

      for (f <- filelist) {
        generatefilebylocation(outputpath, tableName, hdfs, f)
      }
    }
  } catch {
    case e: Exception => {
      println("Exception Occured: " + e)
    }
      sys.exit(99)
  }
  sys.exit(0)
}
